"use strict";

/* eslint-disable react-native/no-inline-styles */
import React, { useState, useEffect } from 'react';
import { View, TextInput, Animated, Platform, TouchableOpacity } from 'react-native';
import Icon from 'react-native-vector-icons/Ionicons';
import Octicons from 'react-native-vector-icons/Octicons';
import { inputStyles as inputStylesheet, inputVariantStyle } from "./Input.style.js";
import { SPACING, WIDTH_WINDOW } from "../../../assets/theme.js";
import { Spacer, Typography } from "../../atom/index.js";
import { useStyles } from 'react-native-unistyles';

/**
 * Component prop
 *
 * @interface InputProps
 * @typedef {InputProps}
 * @extends {TextInputProps}
 */

/**
 * Component
 * @param param0
 * @returns
 */
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
const Input = ({
  keyboardType = 'default',
  variant = 'primary',
  type = 'normal',
  label,
  defaultValue,
  onChangeText,
  style,
  errorText,
  helperText,
  rightIcon,
  leftIcon,
  isPassword = false,
  disabled = false,
  outputRangeSize = 18,
  multiline,
  iIHeight,
  onPress = () => {},
  editable,
  isPressable,
  exLabelStyle,
  exTextInputStyle,
  width,
  placeholderTextColor,
  eyeIconColor,
  alertErrorIcon,
  exContainerStyle,
  ...props
}) => {
  /**
   * States
   */

  const {
    styles: inputStyles,
    theme
  } = useStyles(inputStylesheet);
  const [isFocused, setIsFocused] = useState(false);
  const [animatedIsFocused] = useState(new Animated.Value(defaultValue ? 1 : 0));
  const [text, setText] = useState('');
  const [isPasswordVisible, setIsPasswordVisible] = useState(false);
  iIHeight = iIHeight ?? theme.adaptbyte.length.length_38;

  /**
   * useEffects
   */
  useEffect(() => {
    if (defaultValue) {
      setText(defaultValue);
    } else {
      setText('');
    }
  }, [defaultValue]);
  useEffect(() => {
    Animated.timing(animatedIsFocused, {
      toValue: isFocused || text ? 1 : 0,
      duration: 200,
      useNativeDriver: false
    }).start();
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [isFocused, text]);

  /**
   * Handle text change
   */
  const handleTextChange = val => {
    if (/^\s*$/.test(val)) {
      setText('');
      if (onChangeText) onChangeText('');
    } else {
      setText(val);
      if (onChangeText) onChangeText(val);
    }
  };
  const togglePasswordVisibility = () => {
    setIsPasswordVisible(!isPasswordVisible);
  };

  // Container Style
  const containerStyle = () => {
    const staticStyle = {
      width
    };
    const container = [inputStyles.container, inputVariantStyle(theme)?.[variant]?.container, staticStyle];
    if (errorText && disabled) {
      container.push(inputVariantStyle(theme)?.[variant]?.errorDisabledContainer);
    } else if (disabled) {
      container.push(inputVariantStyle(theme)?.[variant]?.disabledContainer);
    } else if (errorText) {
      container.push(inputVariantStyle(theme)?.[variant]?.errorContainer);
    }
    if (multiline) {
      container.push({
        height: 'auto'
      });
    }
    if (style) {
      container.push(style);
    }
    return container;
  };

  //InputStyle
  const inputStyle = () => {
    const input = [inputStyles.input, inputVariantStyle(theme)?.[variant]?.input, {
      fontFamily: theme.adaptbyte.font.normal.regular
    }];
    let color = inputVariantStyle(theme)?.[variant]?.input?.color;
    if (errorText && disabled) {
      color = theme.adaptbyte.colors.error._500;
    } else if (errorText) {
      color = theme.adaptbyte.colors.danger;
    } else if (disabled) {
      color = inputVariantStyle(theme)?.[variant]?.disabledInput?.color;
    }
    input.push({
      color
    });
    if (leftIcon) {
      input.push({
        left: SPACING.space_16
      });
    }
    if (rightIcon) {
      const contentWidth = width ?? WIDTH_WINDOW;
      const spacing = width ? SPACING.space_72 : SPACING.space_120;
      input.push({
        width: contentWidth - spacing
      });
    }
    if (multiline) {
      input.push({
        height: 'auto'
      });
    }
    if (exTextInputStyle) {
      input.push(exTextInputStyle);
    }
    return input;
  };
  const labelStyle = () => {
    const animatedTop = type !== 'table' ? animatedIsFocused.interpolate({
      inputRange: [0, 1],
      outputRange: [outputRangeSize, 8]
    }) : outputRangeSize;
    const animatedFontSize = type !== 'table' ? animatedIsFocused.interpolate({
      inputRange: [0, 1],
      outputRange: [14, 12]
    }) : 14;
    const animated = {
      position: 'absolute',
      left: 16,
      right: 16,
      display: type === 'table' && text?.length ? 'none' : undefined,
      top: animatedTop,
      fontSize: animatedFontSize,
      fontFamily: theme.adaptbyte.font.normal.regular
    };
    const labels = [inputStyles.label, inputVariantStyle(theme)?.[variant]?.label, animated];
    let color = inputVariantStyle(theme)?.[variant]?.label?.color;
    if (!isFocused && !text?.length && errorText?.length) {
      color = theme.adaptbyte.colors.danger;
    }
    if (leftIcon) {
      labels.push({
        marginLeft: 27
      });
    }
    labels.push({
      color
    });
    if (exLabelStyle) {
      labels.push(exLabelStyle);
    }
    return labels;
  };
  function renderTextInput() {
    if (isPressable) {
      return /*#__PURE__*/_jsx(Typography.Paragraph, {
        text: text,
        style: inputStyle()
      });
    }
    return /*#__PURE__*/_jsx(TextInput, {
      keyboardType: keyboardType,
      editable: editable !== undefined ? editable : !disabled,
      returnKeyType: Platform.OS === 'ios' ? 'done' : 'next',
      defaultValue: defaultValue,
      multiline: multiline,
      style: inputStyle(),
      onChangeText: handleTextChange,
      onFocus: () => setIsFocused(true),
      onBlur: () => setIsFocused(false),
      secureTextEntry: isPassword && !isPasswordVisible,
      placeholderTextColor: placeholderTextColor ?? inputVariantStyle(theme)?.[variant]?.label?.color,
      ...props
    });
  }

  /**
   * Return
   */

  return /*#__PURE__*/_jsxs(View, {
    style: [exContainerStyle],
    children: [/*#__PURE__*/_jsxs(TouchableOpacity, {
      activeOpacity: isPressable && !disabled ? 0 : 1,
      onPress: disabled ? () => {} : onPress,
      style: containerStyle(),
      children: [!!label?.length && /*#__PURE__*/_jsx(Animated.Text, {
        style: labelStyle(),
        children: label
      }), /*#__PURE__*/_jsxs(View, {
        style: [inputStyles.innerInputContainer],
        children: [leftIcon && /*#__PURE__*/_jsx(View, {
          style: inputStyles.leftIconContainer,
          children: leftIcon
        }), /*#__PURE__*/_jsxs(View, {
          style: [inputStyles.innerInput, {
            height: iIHeight,
            minHeight: iIHeight
          }, !label && {
            justifyContent: 'center',
            height: 'auto'
          }, multiline && {
            height: 'auto'
          }],
          children: [label && /*#__PURE__*/_jsx(Spacer, {
            height: type === 'table' ? 9 : 17
          }), renderTextInput()]
        }), isPassword && /*#__PURE__*/_jsx(TouchableOpacity, {
          onPress: togglePasswordVisibility,
          style: inputStyles.iconContainer,
          children: /*#__PURE__*/_jsx(Octicons, {
            name: isPasswordVisible ? 'eye' : 'eye-closed',
            size: 18,
            color: eyeIconColor ?? inputVariantStyle(theme)?.[variant]?.input?.color
          })
        }), rightIcon && /*#__PURE__*/_jsx(View, {
          style: inputStyles.rightIconContainer,
          children: rightIcon
        })]
      })]
    }), errorText || helperText ? /*#__PURE__*/_jsxs(View, {
      style: inputStyles.helperContainer,
      children: [/*#__PURE__*/_jsx(View, {
        style: inputStyles.helperIcon,
        children: errorText && (alertErrorIcon ?? /*#__PURE__*/_jsx(Icon, {
          name: "alert-circle",
          size: 16,
          color: theme.adaptbyte.colors.error._700
        }))
      }), /*#__PURE__*/_jsx(View, {
        style: inputStyles.helperMessage,
        children: /*#__PURE__*/_jsx(Typography.Paragraph, {
          variant: "00",
          weight: "regular",
          text: errorText ? errorText : helperText ?? '',
          color: errorText ? theme.adaptbyte.colors.error._500 : inputVariantStyle(theme)?.[variant]?.label?.color
        })
      })]
    }) : null]
  });
};

/**
 * Export
 */
export default Input;
//# sourceMappingURL=Input.js.map