import React, { useEffect, useState } from 'react';
import { Input } from '../form';
import Feather from 'react-native-vector-icons/Feather';
import { View } from 'react-native';
import { SPACING } from '../../../assets/theme';
import type {
  DateBottomSheetProps,
  InputDateProps,
} from '../../../interfaces/form.interfaces';
import { Button } from '../button';
import { CalendarPicker, TimeslotPicker } from '../picker';
import { Paragraph, Spacer } from '../../atom';
import { BottomSheet } from '../modals';
import { useStyles } from 'react-native-unistyles';

/**
 * Component prop
 *
 * @interface InputDateProps
 * @typedef {InputDateProps}
 * @extends {InputProps}
 */

/**
 * Component
 * @param param0
 * @returns
 */

const DateBottomSheet = ({
  onSelectDate = () => {},
  onSelectTimeslot,
  selectedDate,
  selectedTimeSlot,
  isUseTimeSlot,
  isUsePickerDate = true,
  calendarConfig,
  timeslotConfig = { slots: ['08:00AM - 12:00PM', '02:00PM - 05:00PM'] },
  submitText = 'Continue',
  onSubmit,
}: DateBottomSheetProps) => {
  const [date, setDate] = useState<string | undefined>(selectedDate);
  const [slot, setSlot] = useState<string | undefined>(undefined);

  return (
    <View>
      {isUsePickerDate && (
        <View>
          <CalendarPicker
            dateValue={selectedDate ?? ''}
            onDateChange={setDate}
            {...calendarConfig}
          />
          <Spacer height={SPACING.space_16} />
        </View>
      )}
      {isUseTimeSlot && (
        <View>
          <Paragraph weight="bold" variant="02" text="Time Slot" />
          <Spacer height={SPACING.space_16} />
          <TimeslotPicker
            selectedSlot={selectedTimeSlot}
            onSelect={setSlot}
            {...timeslotConfig}
          />
        </View>
      )}

      <Spacer height={SPACING.space_12} />
      <Button
        disabled={false}
        title={submitText}
        onPress={() => {
          if (typeof onSelectDate === 'function') {
            onSelectDate(date);
          }

          if (typeof onSelectTimeslot === 'function') {
            onSelectTimeslot(slot);
          }

          if (typeof onSubmit === 'function') {
            onSubmit();
          }
        }}
      />
    </View>
  );
};

function InputDate({
  defaultBottomSheetVisible,
  dateValue,
  title,
  config,
  ...props
}: InputDateProps) {
  const [isBottomSheetVisible, setBottomSheetVisible] = useState(false);
  const { theme } = useStyles();

  useEffect(() => {
    if (typeof defaultBottomSheetVisible === 'boolean') {
      setBottomSheetVisible(defaultBottomSheetVisible);
    }
  }, [defaultBottomSheetVisible]);

  return (
    <>
      <Input
        defaultValue={dateValue}
        isPressable
        onPress={() => setBottomSheetVisible(!isBottomSheetVisible)}
        rightIcon={
          <Feather
            name="calendar"
            color={theme.adaptbyte.colors.grayscale._500}
            size={theme.adaptbyte.length.length_20}
          />
        }
        {...props}
      />
      <BottomSheet
        isVisible={isBottomSheetVisible}
        title={title}
        onClosePress={() => setBottomSheetVisible(false)}
      >
        <DateBottomSheet
          {...config}
          onSubmit={() => {
            config?.onSubmit?.();
            setBottomSheetVisible(false);
          }}
        />
      </BottomSheet>
    </>
  );
}

export default InputDate;
