import React, { useState, useRef, useEffect } from 'react';
import { TextInput, View, Platform } from 'react-native';
import { FONTFAMILY } from '../../../assets/theme';
import type { InputOtpProps } from '../../../interfaces';
import { createStyleSheet, useStyles } from 'react-native-unistyles';
import { Typography } from '../../atom';
import Icon from 'react-native-vector-icons/Ionicons';

const InputOtp = ({
  onChangeText,
  style,
  exInputStyle,
  otpLength = 4,
  defaultOtp,
  errorText,
  errorIcon,
}: InputOtpProps) => {
  const [otp, setOTP] = useState<string[]>(Array(otpLength).fill(''));
  const refs = useRef<TextInput[]>([]);
  const { styles, theme } = useStyles(stylesheet);

  // Set Default Otp
  useEffect(() => {
    if (defaultOtp?.length === otpLength) {
      setOTP(defaultOtp);
    }
  }, [defaultOtp, otpLength]);

  const handleOTPChange = (index: number, value: string) => {
    // Allow only numeric values
    if (value !== '' && isNaN(Number(value))) return;
    let newOTP = [...otp];

    if (value?.length > 1) {
      newOTP = [...otp];
      if (index + 1 !== otpLength) {
        newOTP[index + 1] = value?.charAt(1);
      }
      setOTP(newOTP);
      return refs.current[index + 1]?.focus();
    } else {
      newOTP = [...otp];
      newOTP[index] = value;
      setOTP(newOTP);
    }

    // Move focus to the next input if a number is entered
    if (value !== '' && index < otpLength - 1) {
      refs.current[index + 1]?.focus();
    } else if (value === '' && index > 0) {
      // Move focus to the previous input if input is cleared
      refs.current[index - 1]?.focus();
    }

    onChangeText ? onChangeText(newOTP.join('')) : null;
  };

  const handleKeyPress = (index: number, key: string) => {
    // Detect backspace key press
    if (key === 'Backspace' && otp[index] === '' && index > 0) {
      refs.current[index - 1]?.focus();
    }
  };

  return (
    <View>
      <View style={[styles.container, style]}>
        {Array.from({ length: otpLength }).map((_, index) => (
          <TextInput
            key={index}
            ref={(el) => (refs.current[index] = el!)} // Assign ref
            style={[
              styles.input,
              {
                borderColor: errorText?.length
                  ? theme.adaptbyte.colors.error._700
                  : theme.adaptbyte.colors.grayscale._200,
              },
              exInputStyle,
            ]}
            returnKeyType={Platform.OS === 'ios' ? 'done' : 'next'}
            keyboardType="numeric"
            value={otp[index]}
            onChangeText={(value) => handleOTPChange(index, value)}
            onKeyPress={({ nativeEvent }) =>
              handleKeyPress(index, nativeEvent.key)
            }
          />
        ))}
      </View>
      {!!errorText?.length && (
        <View style={styles.errorContainer}>
          {errorIcon ?? (
            <Icon
              name="alert-circle"
              size={16}
              color={theme.adaptbyte.colors.error._700}
            />
          )}
          <Typography.Paragraph
            variant="00"
            weight="medium"
            text={errorText}
            color={theme.adaptbyte.colors.error._500}
          />
        </View>
      )}
    </View>
  );
};

const stylesheet = createStyleSheet((theme) => ({
  container: {
    flexDirection: 'row',
    justifyContent: 'space-between',
    alignItems: 'center',
    marginTop: theme.adaptbyte.spacing.space_16,
  },
  errorContainer: {
    alignSelf: 'center',
    marginTop: theme.adaptbyte.spacing.space_16,
    flexDirection: 'row',
    gap: theme.adaptbyte.spacing.space_10,
  },
  input: {
    height: theme.adaptbyte.length.length_80,
    width: theme.adaptbyte.length.length_72,
    borderWidth: 1,
    marginHorizontal: theme.adaptbyte.spacing.space_5,
    textAlign: 'center',
    borderRadius: theme.adaptbyte.length.length_8,
    backgroundColor: theme.adaptbyte.colors.grayscale._50,
    fontSize: theme.adaptbyte.length.length_48,
    color: theme.adaptbyte.colors.shade.black,
    fontFamily: FONTFAMILY.PlusJakartaSans_medium,
  },
}));

export default InputOtp;
