import {
  View,
  Modal,
  FlatList,
  TouchableOpacity,
  TouchableWithoutFeedback,
  type TouchableOpacityProps,
  type DimensionValue,
} from 'react-native';
import React from 'react';
import Spacer from '../../atom/divider/Spacer';
import Paragraph from '../../atom/typograph/Paragraph';
import Icon from 'react-native-vector-icons/Feather';
import { createStyleSheet, useStyles } from 'react-native-unistyles';

/**
 * Popup List modal props
 */
interface PopupInfoProps {
  isVisible: boolean;
  keyName?: string;
  listData: any[];
  onSelect: (item: any) => void;
  onRequestClose: () => void;
  header?: React.ReactNode;
  bottom?: React.ReactNode;
  left?: (item: any) => React.ReactNode;
  right?: (item: any) => React.ReactNode;
  showCloseIcon?: boolean;
  onPressClose?: () => void;
  itemStyle?: TouchableOpacityProps['style'];
  height?: DimensionValue;
  width?: DimensionValue;
}

/**
 * Popup List Modal
 * @returns {*}
 */
const PopupList = ({
  isVisible = false,
  keyName,
  listData = [],
  onSelect,
  onRequestClose,
  height = 400,
  width = 327,
  showCloseIcon = false,
  onPressClose,
  header,
  bottom,
  left,
  right,
  itemStyle,
}: PopupInfoProps) => {
  const { styles, theme } = useStyles(stylesheet);
  return (
    <Modal animationType="fade" transparent={true} visible={isVisible}>
      <TouchableWithoutFeedback onPress={onRequestClose}>
        <View style={styles.modalContainer}>
          <TouchableWithoutFeedback>
            <View style={[styles.modalView, { height, width }]}>
              {showCloseIcon && (
                <TouchableOpacity
                  style={styles.closeBox}
                  onPress={onPressClose}
                >
                  <Icon name="x" size={24} color={'black'} />
                </TouchableOpacity>
              )}
              {showCloseIcon && (
                <Spacer height={theme.adaptbyte.spacing.space_16} />
              )}
              {header}
              {header && <Spacer height={theme.adaptbyte.spacing.space_16} />}
              {listData?.length > 0 && (
                <FlatList
                  data={listData}
                  ItemSeparatorComponent={
                    (
                      <Spacer height={theme.adaptbyte.spacing.space_16} />
                    ) as any
                  }
                  keyExtractor={(item, index) =>
                    item?.id ? item?.id.toString() : index.toString()
                  }
                  initialNumToRender={10}
                  maxToRenderPerBatch={10}
                  removeClippedSubviews={true}
                  getItemLayout={(_, index) => ({
                    length: 50, // Assuming each item has a fixed height of 50
                    offset: 50 * index,
                    index,
                  })}
                  renderItem={({ item }: any) => {
                    return (
                      <TouchableOpacity
                        style={[styles.itemView, itemStyle]}
                        onPress={() => onSelect(item)}
                      >
                        {left && left(item)}
                        <Paragraph
                          text={keyName ? item[keyName] : item.name}
                          weight="medium"
                          variant="01"
                        />
                        {right && right(item)}
                      </TouchableOpacity>
                    );
                  }}
                />
              )}
              {bottom}
            </View>
          </TouchableWithoutFeedback>
        </View>
      </TouchableWithoutFeedback>
    </Modal>
  );
};

const stylesheet = createStyleSheet((theme) => ({
  closeBox: {
    justifyContent: 'center',
    alignItems: 'flex-end',
    paddingTop: theme.adaptbyte.spacing.space_16,
  },
  modalContainer: {
    flex: 1,
    justifyContent: 'center',
    alignItems: 'center',
    backgroundColor: 'rgba(0,0,0,0.5)',
  },
  modalView: {
    width: 327,
    height: 400,
    backgroundColor: theme.adaptbyte.colors.primaryWhiteHex,
    borderRadius: theme.adaptbyte.borderRadius.radius_8,
    padding: theme.adaptbyte.spacing.space_16,
  },
  itemView: {},
}));

/**
 * Export
 */
export default PopupList;
