import React from 'react';
import {
  StatusBar,
  View,
  Platform,
  KeyboardAvoidingView,
  ScrollView,
} from 'react-native';
import { SafeAreaProvider, SafeAreaView } from 'react-native-safe-area-context';
import { Header } from '../../atom';
import { SPACING } from '../../../assets/theme';
import type { ContainerProps } from '../../../interfaces/templates.interface';
import { createStyleSheet, useStyles } from 'react-native-unistyles';

const statusBarHeight = StatusBar.currentHeight ?? 0;
/**
 * Container prop
 *
 * @interface ContainerProps
 * @typedef {ContainerProps}
 * @extends {SafeAreaViewProps}
 */

/**
 * Container
 *
 * @param {ContainerProps} param0
 * @param {*} param0.children
 * @param {*} param0.style
 * @param {StatusBarStyle} [param0.barStyle="default"]
 * @param {ColorValue} [param0.barBackgroundColor="transparent"]
 * @returns {*}
 */
const Container = ({
  children,
  style,
  barStyle = 'dark-content',
  barBackgroundColor,
  isKeyboardAvoiding = false,
  backgroundColor,
  edges = ['top', 'bottom'],
  barHidden = false,
  FooterComponent,
  headerConfig = {},
  exContentStyle,
  isHeader,
}: ContainerProps) => {
  const { styles, theme } = useStyles(stylesheet);

  backgroundColor = backgroundColor ?? theme.adaptbyte.colors.primaryWhiteHex;
  barBackgroundColor =
    barBackgroundColor ?? theme.adaptbyte.colors.primaryWhiteHex;
  return (
    <SafeAreaProvider>
      <SafeAreaView
        style={[
          style,
          styles.container,
          { backgroundColor: barBackgroundColor },
        ]}
        edges={edges}
      >
        <StatusBar
          translucent
          backgroundColor={barBackgroundColor}
          barStyle={barStyle}
          hidden={barHidden}
        />
        {isHeader && <Header {...headerConfig} />}
        {isKeyboardAvoiding ? (
          <KeyboardAvoidingView
            style={[styles.wrapper, exContentStyle, { backgroundColor }]}
            enabled={isKeyboardAvoiding}
            behavior={Platform.OS === 'ios' ? 'padding' : 'height'}
            keyboardVerticalOffset={
              Platform.OS === 'ios' ? statusBarHeight + SPACING.space_12 : 0
            }
          >
            <ScrollView
              showsVerticalScrollIndicator={false}
              contentContainerStyle={styles.scrollView}
            >
              {children}
            </ScrollView>
          </KeyboardAvoidingView>
        ) : (
          <View style={[styles.wrapper, exContentStyle, { backgroundColor }]}>
            {children}
          </View>
        )}
        {FooterComponent && (
          <View style={{ backgroundColor }}>
            {typeof FooterComponent === 'function' ? (
              <FooterComponent />
            ) : (
              FooterComponent
            )}
          </View>
        )}
      </SafeAreaView>
    </SafeAreaProvider>
  );
};

/**
 * Stylesheet
 *
 * @type {*}
 */
const stylesheet = createStyleSheet((theme) => ({
  container: {
    flexGrow: 1,
  },
  wrapper: {
    paddingTop: theme.adaptbyte.spacing.container.space_top,
    paddingBottom: theme.adaptbyte.spacing.container.space_bottom,
    paddingHorizontal: theme.adaptbyte.spacing.container.space_horizontal,
    flex: 1,
  },
  scrollView: {
    flexGrow: 1,
  },
}));

/**
 * Export container
 */
export default Container;
